@@ -1,4 +1,4 @@
-/* Copyright (C) 2004 Bart
+/* Copyright (C) 2004, 2005, 2006, 2007, 2008 Bart Hakvoort
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -236,7 +236,7 @@ void Dialog_Progress::on_signal_show()
 
 	if ( cancel )
 	{
-		progressbar_current .set_text( _("Operation canceled") ) ;
+		progressbar_current .set_text( _("Operation cancelled") ) ;
 		progressbar_current .set_fraction( 0.0 ) ;
 	}
 	else
@@ -253,7 +253,9 @@ void Dialog_Progress::on_signal_show()
 		str_temp = _("All operations successfully completed") ;
 
 		if ( warnings > 0 )
-			str_temp += " (" + String::ucompose( _("%1 warnings"), warnings ) + ")" ;
+			str_temp += " ("
+			         +  String::ucompose( ngettext("%1 warning", "%1 warnings", warnings), warnings )
+			         +  ")" ;
 
 		progressbar_all .set_text( str_temp ) ;
 		progressbar_all .set_fraction( 1.0 ) ;
@@ -313,7 +315,7 @@ void Dialog_Progress::on_cancel()
 				   Gtk::BUTTONS_NONE,
 				   true ) ;
 		
-	dialog .set_secondary_text( _("Canceling an operation may cause SEVERE filesystem damage.") ) ;
+	dialog .set_secondary_text( _("Canceling an operation might cause SEVERE file system damage.") ) ;
 
 	dialog .add_button( _("Continue Operation"), Gtk::RESPONSE_NONE ) ;
 	dialog .add_button( _("Cancel Operation"), Gtk::RESPONSE_CANCEL ) ;
@@ -342,14 +344,27 @@ void Dialog_Progress::on_save()
 		std::ofstream out( dialog .get_filename() .c_str() ) ;
 		if ( out )
 		{
-			out << "GParted " << VERSION << "<BR><BR>" << std::endl ;
-			out << "Libparted " << signal_get_libparted_version .emit() << "<BR><BR>" << std::endl ; 
+			//Write out proper HTML start
+			out << "<!DOCTYPE html PUBLIC '-//W3C//DTD XHTML 1.0 Transitional//EN' 'http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd'>" << std::endl
+			<< "<html xmlns='http://www.w3.org/1999/xhtml' xml:lang='" << Utils::get_lang() << "'"
+			<< " lang='" << Utils::get_lang() << "'>" << std::endl
+			<< "<head>" << std::endl
+			<< "<meta http-equiv='Content-Type' content='text/html;charset=utf-8' />" << std::endl
+			<< "<title>" << _("GParted Details") << "</title>" << std::endl
+			<< "</head>" << std::endl
+			<< "<body>" << std::endl
+			<< "<p>" << _("GParted") << " " << VERSION << "</p>" << std::endl
+			<< "<p>" << _("Libparted") << " " << signal_get_libparted_version .emit() << "</p>" << std::endl ;
+
+			//Write out each operation
 			for ( unsigned int t = 0 ; t < operations .size() ; t++ )
 			{
 				echo_operation_details( operations[ t ] ->operation_detail, out ) ;
-				out << "<BR>========================================<BR><BR>" << std::endl ;
+				out << "<p>========================================</p>" << std::endl ;
 			}
-				
+
+			//Write out proper HTML finish
+			out << "</body>" << std::endl << "</html>" ;
 			out .close() ;
 		}
 	}
@@ -360,13 +375,13 @@ void Dialog_Progress::echo_operation_details( const OperationDetail & operationd
 	//replace '\n' with '<br>'
 	Glib::ustring temp = operationdetail .get_description() ;
 	for ( unsigned int index = temp .find( "\n" ) ; index < temp .length() ; index = temp .find( "\n" ) )
-		temp .replace( index, 1, "<BR />" ) ;
+		temp .replace( index, 1, "<br />" ) ;
 	
 	//and export everything to some kind of html...
-	out << "<TABLE border=0>" << std::endl ;
-	out << "<TR>" << std::endl ;
-	out << "<TD colspan=2>" << std::endl ;
-	out << temp ;
+	out << "<table border='0'>" << std::endl
+	<< "<tr>" << std::endl
+	<< "<td colspan='2'>" << std::endl
+	<< temp ;
 	if ( ! operationdetail .get_elapsed_time() .empty() )
 		out << "&nbsp;&nbsp;" << operationdetail .get_elapsed_time() ;
 	
@@ -377,19 +392,19 @@ void Dialog_Progress::echo_operation_details( const OperationDetail & operationd
 		switch ( operationdetail .get_status() )
 		{
 			case STATUS_EXECUTE:
-				out << "( EXECUTING )" ;
+				out << "( " << _("EXECUTING") << " )" ;
 				break ;
 			case STATUS_SUCCES:
-				out << "( SUCCESS )" ;
+				out << "( " << _("SUCCESS") << " )" ;
 				break ;
 			case STATUS_ERROR:
-				out << "( ERROR )" ;
+				out << "( " << _("ERROR") << " )" ;
 				break ;
 			case STATUS_INFO:
-				out << "( INFO )" ;
+				out << "( " << _("INFO") << " )" ;
 				break ;
 			case STATUS_N_A:
-				out << "( N/A )" ;
+				out << "( " << _("N/A") << " )" ;
 				break ;
 
 			default:
@@ -397,23 +412,21 @@ void Dialog_Progress::echo_operation_details( const OperationDetail & operationd
 		}
 	}
 	
-	out << std::endl ;
-	out << "</TD>" << std::endl ;
-	out << "</TR>" << std::endl ;
+	out << std::endl << "</td>" << std::endl << "</tr>" << std::endl ;
 	
 	if ( operationdetail .get_childs(). size() )
 	{
-		out << "<TR>" << std::endl ;
-		out << "<TD>&nbsp;&nbsp;&nbsp;&nbsp;</TD>" << std::endl ;
-		out << "<TD>" << std::endl ;
+		out << "<tr>" << std::endl
+		<< "<td>&nbsp;&nbsp;&nbsp;&nbsp;</td>" << std::endl
+		<< "<td>" << std::endl ;
 
 		for ( unsigned int t = 0 ; t <  operationdetail .get_childs() .size() ; t++ )
 			echo_operation_details( operationdetail .get_childs()[ t ], out ) ;
 
-		out << "</TD>" << std::endl ;
-		out << "</TR>" << std::endl ;
+		out << "</td>" << std::endl << "</tr>" << std::endl ;
 	}
-	out << "</TABLE>" << std::endl ;
+	out << "</table>" << std::endl ;
+	
 }
 
 void Dialog_Progress::on_response( int response_id ) 
