@@ -1,4 +1,4 @@
-/* Copyright (C) 2004, 2005, 2006, 2007, 2008 Bart Hakvoort
+/* Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009 Bart Hakvoort
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
  
 #include "../include/Win_GParted.h"
 #include "../include/GParted_Core.h"
+#include "../include/DMRaid.h"
 #include "../include/FS_Info.h"
 #include "../include/OperationCopy.h"
 #include "../include/OperationCreate.h"
@@ -27,6 +28,7 @@
 
 #include "../include/ext2.h"
 #include "../include/ext3.h"
+#include "../include/ext4.h"
 #include "../include/fat16.h"
 #include "../include/fat32.h"
 #include "../include/linux_swap.h"
@@ -87,7 +89,10 @@ void GParted_Core::find_supported_filesystems()
 	
 	ext3 fs_ext3;
 	FILESYSTEMS .push_back( fs_ext3 .get_filesystem_support() ) ;
-	
+
+	ext4 fs_ext4;
+	FILESYSTEMS .push_back( fs_ext4 .get_filesystem_support() ) ;
+
 	fat16 fs_fat16;
 	FILESYSTEMS .push_back( fs_fat16 .get_filesystem_support() ) ;
 	
@@ -105,7 +110,7 @@ void GParted_Core::find_supported_filesystems()
 	
 	linux_swap fs_linux_swap;
 	FILESYSTEMS .push_back( fs_linux_swap .get_filesystem_support() ) ;
-	
+
 	ntfs fs_ntfs;
 	FILESYSTEMS .push_back( fs_ntfs .get_filesystem_support() ) ;
 	
@@ -120,12 +125,29 @@ void GParted_Core::find_supported_filesystems()
 
 	xfs fs_xfs;
 	FILESYSTEMS .push_back( fs_xfs .get_filesystem_support() ) ;
-	
+
+	FS *fs ;
+	//btrfs  FIXME:  Add full support when on-disk-format stabilized
+	fs = new( FS ) ;
+	fs ->filesystem = GParted::FS_BTRFS ;
+	FILESYSTEMS .push_back( * fs ) ;
+
+	//lvm2 physical volume -- not a file system
+	fs = new( FS ) ;
+	fs ->filesystem = GParted::FS_LVM2 ;
+	FILESYSTEMS .push_back( * fs ) ;
+
+	//luks encryption-- not a file system
+	fs = new( FS ) ;
+	fs ->filesystem = GParted::FS_LUKS ;
+	FILESYSTEMS .push_back( * fs ) ;
+
 	//unknown file system (default when no match is found)
-	FS fs ; fs .filesystem = GParted::FS_UNKNOWN ;
-	FILESYSTEMS .push_back( fs ) ;
+	fs = new( FS ) ;
+	fs ->filesystem = GParted::FS_UNKNOWN ;
+	FILESYSTEMS .push_back( * fs ) ;
 }
-	
+
 void GParted_Core::set_user_devices( const std::vector<Glib::ustring> & user_devices ) 
 {
 	this ->device_paths = user_devices ;
@@ -136,6 +158,8 @@ void GParted_Core::set_devices( std::vector<Device> & devices )
 {
 	devices .clear() ;
 	Device temp_device ;
+	FS_Info fs_info( true ) ;  //Refresh cache of file system information
+	DMRaid dmraid( true ) ;    //Refresh cache of dmraid device information
 	
 	init_maps() ;
 	
@@ -151,36 +175,52 @@ void GParted_Core::set_devices( std::vector<Device> & devices )
 		//
 		//try to find all available devices
 		std::ifstream proc_partitions( "/proc/partitions" ) ;
-    	if ( proc_partitions )
-    	{
-    		//parse device names from /proc/partitions
-    		std::string line ;
-    		std::string device ;
-    		while ( getline( proc_partitions, line ) )
-    		{
-    			//Whole disk devices are the ones we want.
-    			//Device names without a trailing digit refer to the whole disk.
-    			device = Utils::regexp_label(line, "^[\t ]+[0-9]+[\t ]+[0-9]+[\t ]+[0-9]+[\t ]+([^0-9]+)$") ;
-    			//Device names that end with a #[^p]# are HP Smart Array Devices (disks)
-    			//E.g., device = /dev/cciss/c0d0, partition = /dev/cciss/c0d0p1
-    			if ( device == "" )
-    				device = Utils::regexp_label(line, "^[\t ]+[0-9]+[\t ]+[0-9]+[\t ]+[0-9]+[\t ]+(.*[0-9]+[^p]{1}[0-9]+)$") ;
-    			if ( device != "" )
-    			{
-    				//try to have libparted detect the device and add it to the list
-    				device = "/dev/" + device;
-    				/*TO TRANSLATORS: looks like Scanning /dev/sda */ 
-    				set_thread_status_message( String::ucompose ( _("Scanning %1"), device ) ) ;
-    				ped_device_get( device .c_str() ) ;
-    			}
-    		}
-    		proc_partitions .close() ;
-    	}
-    	else
-    	{
-    		//file /proc/partitions doesn't exist so use libparted to probe devices
-    		ped_device_probe_all();
-    	}
+		if ( proc_partitions )
+		{
+			//parse device names from /proc/partitions
+			std::string line ;
+			std::string device ;
+			while ( getline( proc_partitions, line ) )
+			{
+				//Whole disk devices are the ones we want.
+				//Device names without a trailing digit refer to the whole disk.
+				device = Utils::regexp_label(line, "^[\t ]+[0-9]+[\t ]+[0-9]+[\t ]+[0-9]+[\t ]+([^0-9]+)$") ;
+				//Recognize /dev/mmcblk* devices.
+				//E.g., device = /dev/mmcblk0, partition = /dev/mmcblk0p1
+				if ( device == "" )
+					device = Utils::regexp_label(line, "^[\t ]+[0-9]+[\t ]+[0-9]+[\t ]+[0-9]+[\t ]+(mmcblk[0-9]+)$") ;
+				//Device names that end with a #[^p]# are HP Smart Array Devices (disks)
+				//E.g., device = /dev/cciss/c0d0, partition = /dev/cciss/c0d0p1
+				if ( device == "" )
+					device = Utils::regexp_label(line, "^[\t ]+[0-9]+[\t ]+[0-9]+[\t ]+[0-9]+[\t ]+(.*[0-9]+[^p]{1}[0-9]+)$") ;
+				if ( device != "" )
+				{
+					//try to have libparted detect the device and add it to the list
+					device = "/dev/" + device;
+					/*TO TRANSLATORS: looks like Scanning /dev/sda */ 
+					set_thread_status_message( String::ucompose ( _("Scanning %1"), device ) ) ;
+					ped_device_get( device .c_str() ) ;
+				}
+			}
+			proc_partitions .close() ;
+
+			//Try to find all dmraid devices
+			if (dmraid .is_dmraid_supported() ) {
+				std::vector<Glib::ustring> dmraid_devices ;
+				dmraid .get_devices( dmraid_devices ) ;
+				for ( unsigned int k=0; k < dmraid_devices .size(); k++ ) {
+					set_thread_status_message( String::ucompose ( _("Scanning %1"), dmraid_devices[k] ) ) ;
+					dmraid .create_dev_map_entries( dmraid_devices[k] ) ;
+					settle_device( 1 ) ;
+					ped_device_get( dmraid_devices[k] .c_str() ) ;
+				}
+			}
+		}
+		else
+		{
+			//file /proc/partitions doesn't exist so use libparted to probe devices
+			ped_device_probe_all();
+		}
 
 		lp_device = ped_device_get_next( NULL );
 		while ( lp_device ) 
@@ -302,7 +342,7 @@ bool GParted_Core::snap_to_cylinder( const Device & device, Partition & partitio
 		} else {
 			diff = partition .sector_start % device .cylsize ;
 		}
-		if ( diff )
+		if ( diff && ! partition .strict_start  )
 		{
 			if ( diff < ( device .cylsize / 2 ) )
 				partition .sector_start -= diff ;
@@ -429,7 +469,15 @@ bool GParted_Core::set_disklabel( const Glib::ustring & device_path, const Glib:
 		
 		close_device_and_disk() ;
 	}
-	
+
+	//delete and recreate disk entries if dmraid
+	DMRaid dmraid ;
+	if ( return_value && dmraid .is_dmraid_device( device_path ) )
+	{
+		dmraid .purge_dev_map_entries( device_path ) ;
+		dmraid .create_dev_map_entries( device_path ) ;
+	}
+
 	return return_value ;	
 }
 	
@@ -538,6 +586,7 @@ void GParted_Core::init_maps()
 	fstab_info .clear() ;
 
 	read_mountpoints_from_file( "/proc/mounts", mount_info ) ;
+	read_mountpoints_from_file_swaps( "/proc/swaps", mount_info ) ;
 	read_mountpoints_from_file( "/etc/mtab", mount_info ) ;
 	read_mountpoints_from_file( "/etc/fstab", fstab_info ) ;
 	
@@ -580,33 +629,54 @@ void GParted_Core::init_maps()
 	}
 }
 
-void GParted_Core::read_mountpoints_from_file( const Glib::ustring & filename,
-					       std::map< Glib::ustring, std::vector<Glib::ustring> > & map ) 
+void GParted_Core::read_mountpoints_from_file(
+	const Glib::ustring & filename,
+	std::map< Glib::ustring, std::vector<Glib::ustring> > & map )
 {
 	std::string line ;
-	char node[4096+1], mountpoint[4096+1] ;
-	unsigned int index ;
+	Glib::ustring node ;
+	Glib::ustring mountpoint ;
+
+	std::ifstream file( filename .c_str() ) ;
+	if ( file )
+	{
+		while ( getline( file, line ) )
+		{
+			node       = Utils::regexp_label( line, "^(/[^ \t]+)[ \t]+[^ \t]+" ) ;
+			mountpoint = Utils::regexp_label( line, "^/[^ \t]+[ \t]+([^ \t]+)" ) ;
+			if ( mountpoint .length() > 0 &&
+			     node .length() > 0 &&
+			     node != "/dev/root" )
+			{
+				//only add this path if it exists
+				if ( file_test( mountpoint, Glib::FILE_TEST_EXISTS ) )
+				{
+					map[ node ] .push_back( mountpoint ) ;
+					//FIXME:  Should check if file is a symbolic link.  If so then find real path
+				}
+			}
+		}
+
+		file .close() ;
+	}
+}
+
+void GParted_Core::read_mountpoints_from_file_swaps(
+	const Glib::ustring & filename,
+	std::map< Glib::ustring, std::vector<Glib::ustring> > & map )
+{
+	std::string line ;
+	std::string node ;
 	
 	std::ifstream file( filename .c_str() ) ;
 	if ( file )
 	{
 		while ( getline( file, line ) )
-			if ( Glib::str_has_prefix( line, "/" ) &&
-			     sscanf( line .c_str(), "%4096s %4096s", node, mountpoint ) == 2 &&
-			     Glib::ustring( node ) != "/dev/root" )
-			{
-				line = mountpoint ;
-				
-				//see if mount point contains spaces and deal with it
-				index = line .find( "\\040" ) ;
-				if ( index < line .length() )
-					line .replace( index, 4, " " ) ;
-				
-				//only add this path if it exists
-                               	if ( file_test( line, Glib::FILE_TEST_EXISTS ) )
-                                       map[ node ] .push_back( line ) ;	
-			}
-			
+		{
+			node = Utils::regexp_label( line, "^(/[^ ]+)" ) ;
+			if ( node .size() > 0 )
+				map[ node ] .push_back( "" /* no mountpoint for swap */ ) ;
+		}
 		file .close() ;
 	}
 }
@@ -626,8 +696,9 @@ void GParted_Core::set_device_partitions( Device & device )
 {
 	int EXT_INDEX = -1 ;
 	char * lp_path ;//we have to free the result of ped_partition_get_path()..
-	FS_Info fs_info ;  //Build cache of file system information
-	
+	FS_Info fs_info ;  //Use cache of file system information
+	DMRaid dmraid ;    //Use cache of dmraid device information
+
 	//clear partitions
 	device .partitions .clear() ;
 
@@ -636,12 +707,31 @@ void GParted_Core::set_device_partitions( Device & device )
 	{
 		libparted_messages .clear() ;
 		partition_temp .Reset() ;
-	
+		bool partition_is_busy = false ;
+
 		switch ( lp_partition ->type )
 		{
 			case PED_PARTITION_NORMAL:
-			case PED_PARTITION_LOGICAL:  
+			case PED_PARTITION_LOGICAL:
 				lp_path = ped_partition_get_path( lp_partition ) ;
+
+				//Handle dmraid devices differently because the minor number might not
+				//  match the last number of the partition filename as shown by "ls -l /dev/mapper"
+				//  This mismatch causes incorrect identification of busy partitions in ped_partition_is_busy(). 
+				if ( dmraid .is_dmraid_device( device .get_path() ) )
+				{
+					//Try device_name + partition_number
+					iter_mp = mount_info .find( device .get_path() + Utils::num_to_str( lp_partition ->num ) ) ;
+					if ( iter_mp != mount_info .end() )
+						partition_is_busy = true ;
+					//Try device_name + p + partition_number
+					iter_mp = mount_info .find( device .get_path() + "p" + Utils::num_to_str( lp_partition ->num ) ) ;
+					if ( iter_mp != mount_info .end() )
+						partition_is_busy = true ;
+				}
+				else
+					partition_is_busy = ped_partition_is_busy( lp_partition ) ;
+
 				partition_temp .Set( device .get_path(),
 						     lp_path,
 						     lp_partition ->num,
@@ -650,7 +740,7 @@ void GParted_Core::set_device_partitions( Device & device )
 						     lp_partition ->geom .start,
 						     lp_partition ->geom .end,
 						     lp_partition ->type,
-						     ped_partition_is_busy( lp_partition ) ) ;
+						     partition_is_busy ) ;
 				free( lp_path ) ;
 						
 				partition_temp .add_paths( get_alternate_paths( partition_temp .get_path() ) ) ;
@@ -663,6 +753,27 @@ void GParted_Core::set_device_partitions( Device & device )
 			
 			case PED_PARTITION_EXTENDED:
 				lp_path = ped_partition_get_path( lp_partition ) ;
+
+				//Handle dmraid devices differently because the minor number might not
+				//  match the last number of the partition filename as shown by "ls -l /dev/mapper"
+				//  This mismatch causes incorrect identification of busy partitions in ped_partition_is_busy(). 
+				if ( dmraid .is_dmraid_device( device .get_path() ) )
+				{
+					for ( unsigned int k = 5; k < 255; k++ )
+					{
+						//Try device_name + [5 to 255]
+						iter_mp = mount_info .find( device .get_path() + Utils::num_to_str( k ) ) ;
+						if ( iter_mp != mount_info .end() )
+							partition_is_busy = true ;
+						//Try device_name + p + [5 to 255]
+						iter_mp = mount_info .find( device .get_path() + "p" + Utils::num_to_str( k ) ) ;
+						if ( iter_mp != mount_info .end() )
+							partition_is_busy = true ;
+					}
+				}
+				else
+					partition_is_busy = ped_partition_is_busy( lp_partition ) ;
+
 				partition_temp .Set( device .get_path(),
 					 	     lp_path, 
 						     lp_partition ->num,
@@ -671,7 +782,7 @@ void GParted_Core::set_device_partitions( Device & device )
 						     lp_partition ->geom .start,
 						     lp_partition ->geom .end,
 						     false,
-						     ped_partition_is_busy( lp_partition ) ) ;
+						     partition_is_busy ) ;
 				free( lp_path ) ;
 				
 				partition_temp .add_paths( get_alternate_paths( partition_temp .get_path() ) ) ;
@@ -724,16 +835,55 @@ void GParted_Core::set_device_partitions( Device & device )
 
 GParted::FILESYSTEM GParted_Core::get_filesystem() 
 {
+	char buf[512] ;
+	char magic[16] ;
+
+	//Check for LUKS encryption prior to libparted file system detection.
+	//  Otherwise encrypted file systems such as ext3 will be detected by
+	//  libparted as 'ext3'.
+
+	//LUKS encryption
+	ped_device_open( lp_device );
+	ped_geometry_read( & lp_partition ->geom, buf, 0, 1 ) ;
+	strncpy(magic, buf+0, 6) ;  magic[6] = '\0' ; //set and terminate string
+	ped_device_close( lp_device );
+
+	if ( Glib::ustring( magic ) == "LUKS\xBA\xBE" )
+	{
+		temp = _( "Linux Unified Key Setup encryption is not yet supported." ) ;
+		temp += "\n" ;
+		partition_temp .messages .push_back( temp ) ;
+		return GParted::FS_LUKS ;
+	}
+
 	//standard libparted file systems..
 	if ( lp_partition && lp_partition ->fs_type )
 	{
 		if ( Glib::ustring( lp_partition ->fs_type ->name ) == "extended" )
 			return GParted::FS_EXTENDED ;
+		else if ( Glib::ustring( lp_partition ->fs_type ->name ) == "btrfs" )
+			return GParted::FS_BTRFS ;
 		else if ( Glib::ustring( lp_partition ->fs_type ->name ) == "ext2" )
 			return GParted::FS_EXT2 ;
 		else if ( Glib::ustring( lp_partition ->fs_type ->name ) == "ext3" )
-			return GParted::FS_EXT3 ;
-		else if ( Glib::ustring( lp_partition ->fs_type ->name ) == "linux-swap" )
+		{
+			//FIXME:  Temporary code to detect ext4.
+			//        Replace when libparted bug #188 "ext4 detected as ext3" is fixed.
+			//        http://parted.alioth.debian.org/cgi-bin/trac.cgi/ticket/188
+			FS_Info fs_info ;
+			temp = fs_info .get_fs_type( Glib::ustring( ped_partition_get_path( lp_partition ) ) ) ; 
+			if ( temp == "ext4" || temp == "ext4dev" )
+				return GParted::FS_EXT4 ;
+			else
+				return GParted::FS_EXT3 ;
+		}
+		else if ( Glib::ustring( lp_partition ->fs_type ->name ) == "ext4" )
+			return GParted::FS_EXT4 ;
+		else if ( Glib::ustring( lp_partition ->fs_type ->name ) == "linux-swap" ||
+		          Glib::ustring( lp_partition ->fs_type ->name ) == "linux-swap(v1)" ||
+		          Glib::ustring( lp_partition ->fs_type ->name ) == "linux-swap(new)" ||
+		          Glib::ustring( lp_partition ->fs_type ->name ) == "linux-swap(v0)" ||
+		          Glib::ustring( lp_partition ->fs_type ->name ) == "linux-swap(old)" )
 			return GParted::FS_LINUX_SWAP ;
 		else if ( Glib::ustring( lp_partition ->fs_type ->name ) == "fat16" )
 			return GParted::FS_FAT16 ;
@@ -757,17 +907,59 @@ GParted::FILESYSTEM GParted_Core::get_filesystem()
 	
 	
 	//other file systems libparted couldn't detect (i've send patches for these file systems to the parted guys)
-	char buf[512] ;
-
-	ped_device_open( lp_device );
+	// - no patches sent to parted for lvm2, or luks
 
 	//reiser4
+	ped_device_open( lp_device );
 	ped_geometry_read( & lp_partition ->geom, buf, 128, 1 ) ;
 	ped_device_close( lp_device );
 	
 	if ( Glib::ustring( buf ) == "ReIsEr4" )
-		return GParted::FS_REISER4 ;		
-		
+		return GParted::FS_REISER4 ;
+
+	//lvm2
+	//NOTE: lvm2 is not a file system but we do wish to recognize the Physical Volume
+	char magic1[16] ;
+	char magic2[16] ;
+
+	ped_device_open( lp_device );
+	ped_geometry_read( & lp_partition ->geom, buf, 1, 1 ) ;
+	strncpy(magic1, buf+0, 8) ;  magic1[8] = '\0' ; //set and terminate string
+	strncpy(magic2, buf+24, 4) ; magic2[4] = '\0' ; //set and terminate string
+	ped_device_close( lp_device );
+
+	if (    Glib::ustring( magic1 ) == "LABELONE"
+		 && Glib::ustring( magic2 ) == "LVM2" )
+	{
+		temp = _( "Logical Volume Management is not yet supported." ) ;
+		temp += "\n" ;
+		partition_temp .messages .push_back( temp ) ;
+		return GParted::FS_LVM2 ;
+	}
+
+	//btrfs
+	const Sector BTRFS_SUPER_INFO_SIZE   = 4096 ;
+	const Sector BTRFS_SUPER_INFO_OFFSET = (64 * 1024) ;
+	const Glib::ustring BTRFS_SIGNATURE  = "_BHRfS_M" ;
+
+	char    buf_btrfs[BTRFS_SUPER_INFO_SIZE] ;
+
+	ped_device_open( lp_device ) ;
+	ped_geometry_read( & lp_partition ->geom, buf_btrfs, \
+	                   (BTRFS_SUPER_INFO_OFFSET / 512), \
+	                   (BTRFS_SUPER_INFO_SIZE / 512)
+	                 ) ;
+	strncpy(magic, buf_btrfs+64, BTRFS_SIGNATURE .size()) ;  magic[BTRFS_SIGNATURE .size()] = '\0' ; //set and terminate string
+	ped_device_close( lp_device ) ;
+
+	if ( magic == BTRFS_SIGNATURE )
+	{
+		temp = _( "BTRFS is not yet supported." ) ;
+		temp += "\n" ;
+		partition_temp .messages .push_back( temp ) ;
+		return GParted::FS_BTRFS ;
+	}
+
 	//no file system found....
 	temp = _( "Unable to detect file system! Possible reasons are:" ) ;
 	temp += "\n-"; 
@@ -852,21 +1044,51 @@ void GParted_Core::insert_unallocated( const Glib::ustring & device_path,
 	
 void GParted_Core::set_mountpoints( std::vector<Partition> & partitions ) 
 {
+	DMRaid dmraid ;	//Use cache of dmraid device information
 	for ( unsigned int t = 0 ; t < partitions .size() ; t++ )
 	{
 		if ( ( partitions[ t ] .type == GParted::TYPE_PRIMARY ||
-		       partitions[ t ] .type == GParted::TYPE_LOGICAL ) &&
-		     partitions[ t ] .filesystem != GParted::FS_LINUX_SWAP )
+		       partitions[ t ] .type == GParted::TYPE_LOGICAL
+		     ) &&
+		     partitions[ t ] .filesystem != GParted::FS_LINUX_SWAP &&
+		     partitions[ t ] .filesystem != GParted::FS_LVM2 &&
+		     partitions[ t ] .filesystem != GParted::FS_LUKS &&
+		     partitions[ t ] .filesystem != GParted::FS_BTRFS
+		   )
 		{
 			if ( partitions[ t ] .busy )
 			{
 				for ( unsigned int i = 0 ; i < partitions[ t ] .get_paths() .size() ; i++ )
 				{
-					iter_mp = mount_info .find( partitions[ t ] .get_paths()[ i ] ) ;
-					if ( iter_mp != mount_info .end() )
+					//Handle dmraid devices differently because there may be more
+					//  than one partition name.
+					//  E.g., there might be names with and/or without a 'p' between
+					//        the device name and partition number.
+					if ( dmraid .is_dmraid_device( partitions[ t ] .device_path ) )
 					{
-						partitions[ t ] .add_mountpoints( iter_mp ->second ) ;
-						break ;
+						//Try device_name + partition_number
+						iter_mp = mount_info .find( partitions[ t ] .device_path + Utils::num_to_str( partitions[ t ] .partition_number ) ) ;
+						if ( iter_mp != mount_info .end() )
+						{
+							partitions[ t ] .add_mountpoints( iter_mp ->second ) ;
+							break ;
+						}
+						//Try device_name + p + partition_number
+						iter_mp = mount_info .find( partitions[ t ] .device_path + "p" + Utils::num_to_str( partitions[ t ] .partition_number ) ) ;
+						if ( iter_mp != mount_info .end() )
+						{
+							partitions[ t ] .add_mountpoints( iter_mp ->second ) ;
+							break ;
+						}
+					}
+					else
+					{
+						iter_mp = mount_info .find( partitions[ t ] .get_paths()[ i ] ) ;
+						if ( iter_mp != mount_info .end() )
+						{
+							partitions[ t ] .add_mountpoints( iter_mp ->second ) ;
+							break ;
+						}
 					}
 				}
 
@@ -896,7 +1118,11 @@ void GParted_Core::set_used_sectors( std::vector<Partition> & partitions )
 	for ( unsigned int t = 0 ; t < partitions .size() ; t++ )
 	{
 		if ( partitions[ t ] .filesystem != GParted::FS_LINUX_SWAP &&
-		     partitions[ t ] .filesystem != GParted::FS_UNKNOWN )
+		     partitions[ t ] .filesystem != GParted::FS_BTRFS &&
+		     partitions[ t ] .filesystem != GParted::FS_LUKS &&
+		     partitions[ t ] .filesystem != GParted::FS_LVM2 &&
+		     partitions[ t ] .filesystem != GParted::FS_UNKNOWN
+		   )
 		{
 			if ( partitions[ t ] .type == GParted::TYPE_PRIMARY ||
 			     partitions[ t ] .type == GParted::TYPE_LOGICAL ) 
@@ -1084,16 +1310,16 @@ bool GParted_Core::create_partition( Partition & new_partition, OperationDetail
 		close_device_and_disk() ;
 	}
 
-	if ( new_partition .partition_number > 0 && erase_filesystem_signatures( new_partition ) )
-	{ 
-		operationdetail .get_last_child() .set_status( STATUS_SUCCES ) ;
-		return true ;
-	}
-	else
-	{		
-		operationdetail .get_last_child() .set_status( STATUS_ERROR ) ;	
-		return false ;
-	} 
+	bool succes = new_partition .partition_number > 0 && erase_filesystem_signatures( new_partition ) ;
+
+	//create dev map entries if dmraid
+	DMRaid dmraid ;
+	if ( succes && dmraid .is_dmraid_device( new_partition .device_path ) )
+		succes = dmraid .create_dev_map_entries( new_partition, operationdetail .get_last_child() ) ;
+
+	operationdetail .get_last_child() .set_status( succes ? STATUS_SUCCES : STATUS_ERROR ) ;
+
+	return succes ;
 }
 	
 bool GParted_Core::create_filesystem( const Partition & partition, OperationDetail & operationdetail ) 
@@ -1148,6 +1374,23 @@ bool GParted_Core::Delete( const Partition & partition, OperationDetail & operat
 		close_device_and_disk() ;
 	}
 
+	//delete partition dev mapper entry, and delete and recreate all other affected dev mapper entries if dmraid
+	DMRaid dmraid ;
+	if ( succes && dmraid .is_dmraid_device( partition .device_path ) )
+	{
+		//Open disk handle before and close after to prevent application crash.
+		if ( open_device_and_disk( partition .device_path ) )
+		{
+			if ( ! dmraid .delete_affected_dev_map_entries( partition, operationdetail .get_last_child() ) )
+				succes = false ;	//comand failed
+
+			if ( ! dmraid .create_dev_map_entries( partition, operationdetail .get_last_child() ) )
+				succes = false ;	//command failed
+
+			close_device_and_disk() ;
+		}
+	}
+
 	operationdetail .get_last_child() .set_status( succes ? STATUS_SUCCES : STATUS_ERROR ) ;
 	return succes ;
 }
@@ -1193,7 +1436,10 @@ bool GParted_Core::resize_move( const Device & device,
 			  	Partition & partition_new,
 			  	OperationDetail & operationdetail ) 
 {
-	if ( partition_new .strict || calculate_exact_geom( partition_old, partition_new, operationdetail ) )
+	if (   partition_new .strict
+		|| partition_new .strict_start
+		|| calculate_exact_geom( partition_old, partition_new, operationdetail )
+	   )
 	{
 		if ( partition_old .type == TYPE_EXTENDED )
 			return resize_move_partition( partition_old, partition_new, operationdetail ) ;
@@ -1250,8 +1496,8 @@ bool GParted_Core::move( const Device & device,
 		//(maybe i should do some reading on how non-msdos disklabels deal with metadata....)
 		if ( partition_new .sector_start < partition_old .sector_start )
 		{
-	       		if ( resize_move_partition( partition_old, partition_new, operationdetail ) )
-	       	   	{
+			if ( resize_move_partition( partition_old, partition_new, operationdetail ) )
+			{
 				if ( ! move_filesystem( partition_old, partition_new, operationdetail ) )
 				{
 					operationdetail .add_child( OperationDetail( _("rollback last change to the partition table") ) ) ;
@@ -1266,13 +1512,13 @@ bool GParted_Core::move( const Device & device,
 			}
 		}
 		else
-	       		succes = move_filesystem( partition_old, partition_new, operationdetail ) &&
-	       			 resize_move_partition( partition_old, partition_new, operationdetail ) ;
+			succes = move_filesystem( partition_old, partition_new, operationdetail ) &&
+				resize_move_partition( partition_old, partition_new, operationdetail ) ;
 
 		succes = succes &&
-			 update_bootsector( partition_new, operationdetail ) &&
-	       		 check_repair_filesystem( partition_new, operationdetail ) &&
-	       		 maximize_filesystem( partition_new, operationdetail ) ;
+			update_bootsector( partition_new, operationdetail ) &&
+			check_repair_filesystem( partition_new, operationdetail ) &&
+			maximize_filesystem( partition_new, operationdetail ) ;
 	}
 
 	return succes ;
@@ -1532,10 +1778,14 @@ bool GParted_Core::resize_move_partition( const Partition & partition_old,
 		
 		if ( lp_partition )
 		{
-			PedGeometry *geom = ped_geometry_new( lp_device,
-							      partition_new .sector_start,
-							      partition_new .get_length() ) ;
-			constraint = ped_constraint_exact( geom ) ;
+			if ( partition_new .strict || partition_new .strict_start ) {
+				PedGeometry *geom = ped_geometry_new( lp_device,
+									  partition_new .sector_start,
+									  partition_new .get_length() ) ;
+				constraint = ped_constraint_exact( geom ) ;
+			}
+			else
+				constraint = ped_constraint_any( lp_device ) ;
 
 			if ( constraint )
 			{
@@ -1569,6 +1819,18 @@ bool GParted_Core::resize_move_partition( const Partition & partition_old,
 					  	  Utils::format_size( new_end - new_start + 1 ) ),
 			STATUS_NONE, 
 			FONT_ITALIC ) ) ;
+
+		//update dev mapper entry if partition is dmraid.
+		DMRaid dmraid ;
+		if ( return_value && dmraid .is_dmraid_device( partition_new .device_path ) )
+		{
+			//Open disk handle before and close after to prevent application crash.
+			if ( open_device_and_disk( partition_new .device_path ) )
+			{
+				return_value = dmraid .update_dev_map_entry( partition_new, operationdetail .get_last_child() ) ;
+				close_device_and_disk() ;
+			}
+		}
 	}
 	
 	operationdetail .get_last_child() .set_status( return_value ? STATUS_SUCCES : STATUS_ERROR ) ;
@@ -1918,7 +2180,16 @@ bool GParted_Core::set_partition_type( const Partition & partition, OperationDet
 		PedFileSystemType * fs_type = 
 			ped_file_system_type_get( Utils::get_filesystem_string( partition .filesystem ) .c_str() ) ;
 
+		//If not found, and FS is linux-swap, then try linux-swap(v1)
+		if ( ! fs_type && Utils::get_filesystem_string( partition .filesystem ) == "linux-swap" )
+			fs_type = ped_file_system_type_get( "linux-swap(v1)" ) ;
+
+		//If not found, and FS is linux-swap, then try linux-swap(new)
+		if ( ! fs_type && Utils::get_filesystem_string( partition .filesystem ) == "linux-swap" )
+			fs_type = ped_file_system_type_get( "linux-swap(new)" ) ;
+
 		//default is Linux (83)
+		//FIXME:  Currently HFS+ and HFS get set to 83, but should be AF.
 		if ( ! fs_type )
 			fs_type = ped_file_system_type_get( "ext2" ) ;
 
@@ -2226,6 +2497,18 @@ bool GParted_Core::calculate_exact_geom( const Partition & partition_old,
 					  	  Utils::format_size( partition_new .get_length() ) ),
 			STATUS_NONE,
 			FONT_ITALIC ) ) ;
+
+		//Update dev mapper entry if partition is dmraid.
+		DMRaid dmraid ;
+		if ( succes && dmraid .is_dmraid_device( partition_new .device_path ) )
+		{
+			//Open disk handle before and close after to prevent application crash.
+			if ( open_device_and_disk( partition_new .device_path ) )
+			{
+				succes = dmraid .update_dev_map_entry( partition_new, operationdetail .get_last_child() ) ;
+				close_device_and_disk() ;
+			}
+		}
 	}
 
 	operationdetail .get_last_child() .set_status( succes ? STATUS_SUCCES : STATUS_ERROR ) ;
@@ -2241,6 +2524,7 @@ bool GParted_Core::set_proper_filesystem( const FILESYSTEM & filesystem )
 	{
 		case FS_EXT2		: p_filesystem = new ext2() ;	 	break ;
 		case FS_EXT3		: p_filesystem = new ext3() ; 		break ;
+		case FS_EXT4		: p_filesystem = new ext4() ; 		break ;
 		case FS_LINUX_SWAP	: p_filesystem = new linux_swap() ; 	break ;
 		case FS_FAT16		: p_filesystem = new fat16() ; 		break ;
 		case FS_FAT32		: p_filesystem = new fat32() ; 		break ;
@@ -2288,37 +2572,83 @@ bool GParted_Core::erase_filesystem_signatures( const Partition & partition )
 bool GParted_Core::update_bootsector( const Partition & partition, OperationDetail & operationdetail ) 
 {
 	//only for ntfs atm...
-	//FIXME: 1) this should be done without relying on external commands
-	//	 2) this should probably be done in the fsclasses...
+	//FIXME: this should probably be done in the fs classes...
 	if ( partition .filesystem == FS_NTFS )
 	{
+		//The NTFS file system stores a value in the boot record called the
+		//  Number of Hidden Sectors.  This value must match the partition start
+		//  sector number in order for Windows to boot from the file system.
+		//  For more details, refer to the NTFS Volume Boot Record at:
+		//  http://www.geocities.com/thestarman3/asm/mbr/NTFSBR.htm
+
 		operationdetail .add_child( OperationDetail( 
-			String::ucompose( _("updating boot sector of %1 file system on %2"),
+				/*TO TRANSLATORS: update boot sector of ntfs file system on /dev/sdd1 */
+				String::ucompose( _("update boot sector of %1 file system on %2"),
 					  Utils::get_filesystem_string( partition .filesystem ),
 					  partition .get_path() ) ) ) ;
 
+		//convert start sector to hex string
 		std::stringstream ss ;
 		ss << std::hex << partition .sector_start ;
 		Glib::ustring hex = ss .str() ;
-	
+
 		//fill with zeros and reverse...
 		hex .insert( 0, 8 - hex .length(), '0' ) ;
 		Glib::ustring reversed_hex ;
 		for ( int t = 6 ; t >= 0 ; t -=2 )
 			reversed_hex .append( hex .substr( t, 2 ) ) ;
 
-		Glib::ustring output, error, command ;
-		command = 
-			"echo " + reversed_hex + " | /usr/bin/xxd -r -p | /bin/dd conv=notrunc of=" + partition .get_path() + " bs=1 seek=28" ;
+		//convert reversed hex codes into ascii characters
+		char buf[4] ;
+		for ( unsigned int k = 0; (k < 4 && k < (reversed_hex .length() / 2)); k++ )
+		{
+			Glib::ustring tmp_hex = "0x" + reversed_hex .substr( k * 2, 2 ) ;
+			buf[k] = (char)( std::strtol( tmp_hex .c_str(), NULL, 16 ) ) ;
+		}
 
-		operationdetail .get_last_child() .add_child( OperationDetail( command, STATUS_NONE, FONT_BOLD_ITALIC ) ) ;
-		bool succes = ! Utils::execute_command( command, output, error ) ;
+		//write new Number of Hidden Sectors value into NTFS boot sector at offset 0x1C
+		Glib::ustring error_message = "" ;
+		std::ofstream dev_file ;
+		dev_file .open( partition .get_path() .c_str(), std::ios::out | std::ios::binary ) ;
+		if ( dev_file .is_open() )
+		{
+			dev_file .seekp( 0x1C ) ;
+			if ( dev_file .good() )
+			{
+				dev_file .write( buf, 4 ) ;
+				if ( dev_file .bad() )
+				{
+					/*TO TRANSLATORS: looks like Error trying to write to boot sector in /dev/sdd1 */
+					error_message = String::ucompose( _("Error trying to write to boot sector in %1"), partition .get_path() ) ;
+				}
+			}
+			else
+			{
+				/*TO TRANSLATORS: looks like Error trying to seek to position 0x1C in /dev/sdd1 */
+				error_message = String::ucompose( _("Error trying to seek to position 0x1c in %1"), partition .get_path() ) ;
+			}
+			dev_file .close( ) ;
+		}
+		else
+		{
+			/*TO TRANSLATORS: looks like Error trying to open /dev/sdd1 */
+			error_message = String::ucompose( _("Error trying to open %1"), partition .get_path() ) ;
+		}
 
-		if ( ! output .empty() )
-			operationdetail .get_last_child() .get_last_child() .add_child( OperationDetail( output, STATUS_NONE, FONT_ITALIC ) ) ;
-		
-		if ( ! error .empty() )
-			operationdetail .get_last_child() .get_last_child() .add_child( OperationDetail( error, STATUS_NONE, FONT_ITALIC ) ) ;
+		//append error messages if any found
+		bool succes = true ;
+		if ( ! error_message .empty() )
+		{
+			succes = false ;
+			error_message += "\n" ;
+			/*TO TRANSLATORS: looks like Failed to set the number of hidden sectors to 05ab4f00 in the ntfs boot record. */
+			error_message += String::ucompose( _("Failed to set the number of hidden sectors to %1 in the ntfs boot record."), reversed_hex ) ;
+			error_message += "\n" ;
+			error_message += String::ucompose( _("You might try the following command to correct the problem:"), reversed_hex ) ;
+			error_message += "\n" ;
+			error_message += String::ucompose( "echo %1 | xxd -r -p | dd conv=notrunc of=%2 bs=1 seek=28", reversed_hex, partition .get_path() ) ;
+			operationdetail .get_last_child() .add_child( OperationDetail( error_message, STATUS_NONE, FONT_ITALIC ) ) ;
+		}
 
 		operationdetail .get_last_child() .set_status( succes ? STATUS_SUCCES : STATUS_ERROR ) ;
 		return succes ;
@@ -2383,16 +2713,28 @@ bool GParted_Core::commit()
 
 bool GParted_Core::commit_to_os( std::time_t timeout ) 
 {
-	bool succes = ped_disk_commit_to_os( lp_disk ) ;
-
-	if ( Glib::find_program_in_path( "udevsettle" ) .empty() )
-		sleep( timeout ) ;
+	DMRaid dmraid ;
+	bool succes ;
+	if ( dmraid .is_dmraid_device( lp_disk ->dev ->path ) )
+		succes = true ;
 	else
-		Utils::execute_command( "udevsettle --timeout=" + Utils::num_to_str( timeout ) ) ;
+		succes = ped_disk_commit_to_os( lp_disk ) ;
+
+	settle_device( timeout ) ;
 
 	return succes ;
 }
-	
+
+void GParted_Core::settle_device( std::time_t timeout )
+{
+	if ( ! Glib::find_program_in_path( "udevsettle" ) .empty() )
+		Utils::execute_command( "udevsettle --timeout=" + Utils::num_to_str( timeout ) ) ;
+	else if ( ! Glib::find_program_in_path( "udevadm" ) .empty() )
+		Utils::execute_command( "udevadm settle --timeout=" + Utils::num_to_str( timeout ) ) ;
+	else
+		sleep( timeout ) ;
+}
+
 PedExceptionOption GParted_Core::ped_exception_handler( PedException * e ) 
 {
         std::cout << e ->message << std::endl ;
